## RangeShifts_code.R for Laughlin and Pomara (2023)
## Ornithology.

## I requested CBC data from the National Audubon Society
## for 65 species of songbirds and woodpeckers.
## The data comes in two different files: the raw circle
## files and the effort files. The code below imports the 
## files and calculates the metric of interest: # birds per 
## party hour. From here, we calculate the mean geographic
## latitude for each year in the study period (1980 - 2019),
## and run linear regressions for each species across these
## years for ENA, ATL, and MISS individually. The raw data
## from the CBC that we used is uploaded to Dryad in 
## batches. The code below anlyzes batch 2 as an example.

## import two CBC datafiles: raw circle count data and effort data
## I uploaded these files in batches to Dryad, and this is an example
## using one of the file batches there.

setwd("G:/My Drive/UNCA/Research/Range Shifts/New_submission/Ornithology submission/Revision 2/Ornithology_FinalVersions/Data_Dryad")
cbc1 <- read.csv('cbc_batch6_1Spec_Circle.csv')
eff1 <- read.csv('cbc_batch6_1Spec_effort.csv')

## Remove pre-1980 data (some of these files already have)
cbc1 <- cbc1[which(cbc1$Count_yr > 79),]
eff1 <- eff1[which(eff1$Count_yr > 79),]

## Divide into ATL and MISS flyway by state,
## then remove all data outside these states.
state <- sapply(strsplit(as.character(cbc1$Subnational_code), split='-', fixed=TRUE), function(x) (x[2]))
cbc1$state <- state
atl <- c('FL', 'GA', 'SC', 'NC', 'VA', 'WV', 'PA', 'MD',
         'DE', 'RI', 'NJ', 'NY', 'CT', 'VT', 'NH', 'ME',
         'MA', 'NS', 'NB', 'QC', 'PE', 'NL', 'fl', 'ga',
         'sc', 'nc', 'va', 'wv', 'pa', 'md', 'de', 'rh',
         'nj', 'ny', 'ct', 'vt', 'nh', 'me', 'ma', 'ns',
         'nb', 'qc', 'pe', 'nl')
miss <- c('LA', 'MS', 'AL', 'AR', 'TN', 'KY', 'MO', 'OH',
          'IL', 'IN', 'IA', 'MI', 'WI', 'MN', 'ON', 'MB',
          'la', 'ms', 'al', 'ar', 'tn', 'ky', 'mo', 'oh',
          'il', 'in', 'ia', 'mi', 'wi', 'mn', 'on', 'mb')

cbc1_atl <- NULL
for (i in 1:length(atl)){
  my_atl <- cbc1[which(cbc1$state==atl[i]),]
  cbc1_atl <- rbind(cbc1_atl, my_atl)
}
cbc1_miss <- NULL
for (i in 1:length(miss)){
  my_miss <- cbc1[which(cbc1$state==miss[i]),]
  cbc1_miss <- rbind(cbc1_miss, my_miss)
}

cbc1 <- rbind(cbc1_atl, cbc1_miss)


## Calculate effort (bph = # Birds per Party Hour)
## Change $Hours to numeric, remove comma (1,375.00)
eff2 <- eff1
eff2$Hours <- as.numeric(gsub(",","", eff2$Hours))

## Duplicate counts need to be deleted from main Circle file:
cbc2 <- NULL     # this creates a new blank variable to which you add
my_spec2 <- NULL
min_year <- as.numeric(min(cbc1$Count_yr))
max_year <- as.numeric(max(cbc1$Count_yr))
uniq_spec <- as.character(unique(cbc1$COM_NAME))

## remove duplicate counts
for (j in 1:length(uniq_spec)){
  my_spec <- uniq_spec[j]
  my_spec_data <- cbc1[which(cbc1$COM_NAME==my_spec),]
  for (i in min_year:max_year){
    my.data <- my_spec_data[which(my_spec_data$Count_yr==i),]
    my.dup <- which(duplicated(my.data$Abbrev))
    if (length(my.dup)>0){
      my.data <- my.data[-my.dup,]
    }
    my_spec2 <- rbind(my_spec2, my.data)
  }
  cbc2 <- rbind(cbc2, my_spec2)
  my_spec2 <- NULL
}

## change how_many to numeric and remove commas:
cbc2$how_many <- as.numeric(gsub(",","", cbc2$how_many))

## add up the hours for each count (car + walk + bike, etc)
ph <- NULL      # ph = party hours
uniq <- unique(eff2$Count_yr)
for (i in uniq[1:length(uniq)]){
  gh <- eff2[eff2$Count_yr==i,]
  my.ph <- aggregate(gh$Hours, by=list(gh$Abbrev), sum)
  my.ph$Count_yr <- i
  ph <- rbind(ph,my.ph)
}
names(ph) <- c("Abbrev", "Hours", "Count_yr")

## add the party hours column to cbc2 file:
cbc3 <- NULL
for (j in 1:length(uniq_spec)){
  my_spec <- uniq_spec[j]
  my_spec_data <- cbc2[which(cbc2$COM_NAME==my_spec),]
  for (i in 1:length(uniq)){
    my.data <- my_spec_data[which(my_spec_data$Count_yr==uniq[i]),]
    ph.year <- ph[which(ph$Count_yr==uniq[i]),]
    gh <- merge(my.data, ph.year, by="Abbrev")
    cbc3 <- rbind(cbc3, gh)
  }
  gh <- NULL
}

## remove rows with either 0 birds or 0 party hours:
cbc3 <- cbc3[-which(cbc3$how_many=='0'),]
cbc3 <- cbc3[-which(cbc3$Hours=='0'),]
## remove rows with > 250 party hours to remove errors/outliers
cbc4 <- cbc3[which(cbc3$Hours < 250),]

## create new column, bph, birds per party hour
cbc4$bph <- cbc4$how_many / cbc4$Hours

## add sp_code column
com_name <- as.character(unique(cbc4$COM_NAME))

## create vector of 4-letter code, depending on file
#sp_code <- "EATO"
#sp_code <- "BHVI"
#sp_code <- "DEJU"
#sp_code <- c("YRWA", "PAWA", "RBWO", "WCSP", "CORA")
#sp_code <- c("COGR", "NOFL", "WIWR")
#sp_code <- c("SEWR", "RHWO", "MAWR")
#sp_code <- c("FICR", "TRES", "COYE", "EAME", "EAPH",
#             "HOWR", "BGGN", "PIWA", "CHSP", "SAVS",
#             "RCKI", "YBSA", "HETH", "BRTH", "SWSP",
#             "BHNU", "RUBL", "CACH", "BRCR", "FOSP")
sp_code <- c("AMRO", "RWBL", "NOMO", "BLJA", "NOCA",
              "GRCA", "CARW", "BHCO", "AMCR",
             "PIWO", "AMGO", "WEVI", "DOWO", "OCWA",
             "CEDW", "SOSP", "EABL", "HAWO", "VESP",
              "WTSP", "TUTI", "FISP", "PUFI", "GCKI",
              "AMPI", "WBNU", "HOLA", "RBNU", "PISI",
              "HOFI", "ATSP", "BCCH")

sp_codes <- data.frame(COM_NAME=com_name, sp_code=sp_code)

cbc5 <- merge(sp_codes, cbc4, by="COM_NAME")
cbc6 <- cbc5[,-c(7,13,14,15,18)]

## This file (cbc6) now contains the data necessary
## to calculate the geographic mean (LCA) for each 
## year, across ENA and for each flyway.


###########################
### calculate geog mean ###

library('geosphere')
cbc_80 <- cbc6

## To minimize spatial bias involved when new CBC circles come 
## online, we limited the number of circles to only
## those that have contributed at least 36 years of data 
## out of our 40-year study period (>= 90%)

## limit to "good" sites (sites_80 with > 35 years)
min_year <- min(cbc_80$Count_yr.x)
max_year <- max(cbc_80$Count_yr.x)
uniq_abbrev <- as.character(unique(cbc_80$Abbrev))
uniq_80 <- data.frame(abbrev=uniq_abbrev)
uniq_lon <- NULL
uniq_lat <- NULL

for (i in 1:length(uniq_abbrev)){
  uniq_lon[i] <- cbc_80$Longitude[which(cbc_80$Abbrev==uniq_abbrev[i])][1]
  uniq_lat[i] <- cbc_80$Latitude[which(cbc_80$Abbrev==uniq_abbrev[i])][1]
}

uniq_80 <- data.frame(Abbrev=uniq_abbrev, Longitude=uniq_lon,
                      Latitude=uniq_lat)

num_years <- NULL
for (j in 1:length(uniq_abbrev)){
  my_data <- cbc_80[which(cbc_80$Abbrev==uniq_abbrev[j]),]
  num_years[j] <- length(unique(my_data$Count_yr.x))
}

uniq_80_good <- uniq_abbrev[which(num_years>36)]

cbc_80_good <- NULL
for (i in 1:length(uniq_80_good)){
  my_data <- cbc_80[which(cbc_80$Abbrev==uniq_80_good[i]),]
  cbc_80_good <- rbind(cbc_80_good, my_data)
}
cbc <- cbc_80_good

min_year <- min(cbc$Count_yr.x)
max_year <- max(cbc$Count_yr.x)
my_cbc <- data.frame(Count_yr=min_year:max_year)
uniq_spec <- as.character(unique(cbc$COM_NAME))
atl <- c('FL', 'GA', 'SC', 'NC', 'VA', 'WV', 'PA', 'MD',
         'DE', 'RI', 'NJ', 'NY', 'CT', 'VT', 'NH', 'ME',
         'MA', 'NS', 'NB', 'QC', 'PE', 'NL', 'fl', 'ga',
         'sc', 'nc', 'va', 'wv', 'pa', 'md', 'de', 'rh',
         'nj', 'ny', 'ct', 'vt', 'nh', 'me', 'ma', 'ns',
         'nb', 'qc', 'pe', 'nl')
miss <- c('LA', 'MS', 'AL', 'AR', 'TN', 'KY', 'MO', 'OH',
          'IL', 'IN', 'IA', 'MI', 'WI', 'MN', 'ON', 'MB',
          'la', 'ms', 'al', 'ar', 'tn', 'ky', 'mo', 'oh',
          'il', 'in', 'ia', 'mi', 'wi', 'mn', 'on', 'mb')

#For each species:
for (j in 1:length(uniq_spec)){
  my_spec <- cbc[which(cbc$COM_NAME==uniq_spec[j]),]
  my_sp_mean <- NULL
  my_sp_mean_atl <- NULL
  my_sp_mean_miss <- NULL
  
  
  #For each year:
  for (i in min_year:max_year){
    my_data <- my_spec[which(my_spec$Count_yr.x==i),]
    
    my_data_atl <- NULL
    for (k in 1:length(atl)){
      my_atl <- my_data[which(my_data$state==atl[k]),]
      my_data_atl <- rbind(my_data_atl, my_atl)
    }
    my_data_miss <- NULL
    for (k in 1:length(miss)){
      my_miss <- my_data[which(my_data$state==miss[k]),]
      my_data_miss <- rbind(my_data_miss, my_miss)
    }
    
    #calculate overall mean latitude
    xy <- cbind(x=my_data$Longitude, y=my_data$Latitude)
    my_mean2 <- geomean(xy, w=my_data$bph)
    my_sp_mean <- rbind(my_sp_mean, as.numeric(my_mean2[,2]))
    
    #calculate atlantic mean
    xy_atl <- cbind(x=my_data_atl$Longitude, y=my_data_atl$Latitude)
    my_mean_atl <- geomean(xy_atl, w=my_data_atl$bph)
    my_sp_mean_atl <- rbind(my_sp_mean_atl, as.numeric(my_mean_atl[,2]))
    
    #calculate mississippi mean
    xy_miss <- cbind(x=my_data_miss$Longitude, y=my_data_miss$Latitude)
    my_mean_miss <- geomean(xy_miss, w=my_data_miss$bph)
    my_sp_mean_miss <- rbind(my_sp_mean_miss, as.numeric(my_mean_miss[,2]))
    
  }
  my_df <- data.frame(a=my_sp_mean, b=my_sp_mean_atl, c=my_sp_mean_miss)
  
  names(my_df) <- c(paste(as.character(my_spec$sp_code[j]), "mean_lat", sep='_'),
                    paste(as.character(my_spec$sp_code[j]), "mean_lat_atl", sep='_'),
                    paste(as.character(my_spec$sp_code[j]), "mean_lat_miss", sep='_'))
  
  my_cbc <- cbind(my_cbc, my_df)
  
}

## This file (my_cbc) now contains the geographic mean
## for each species for each year of the study period. This
## file is uploaded to Dryad called "LCA_65species.csv", and
## will be used in the next .R file to calculate the Pearson's
## correlation between flyways and geographic mean at the start
## of the study period (1980-1982). 

## The next step is to calculate the slope of the linear
## regression models for LCA ~ year for ENA, ATL, and MISS
## for each species.


############################
###   calculate slope    ###

## We ran linear regressions using year as the explanatory
## variable and LCA as the response variable. The following
## code calculates the slope of these regressions for each
## species and each spatial area (ENA, ATL, and MISS)

cbc <- my_cbc

## calculate all slopes, significant or not (P =/< 0.05)
sp_codes <- unique(sapply(strsplit(as.character(names(cbc[,2:ncol(cbc)])), split='_', fixed=TRUE), function(x) (x[1])))

cbc_slopes <- data.frame(species=sp_codes, ena_sig=rep(0, length(sp_codes)),
                         ena_all=rep(0, length(sp_codes)),
                         atl_sig=rep(0, length(sp_codes)),
                         atl_all=rep(0, length(sp_codes)),
                         miss_sig=rep(0, length(sp_codes)),
                         miss_all=rep(0, length(sp_codes)))

for (i in 1:length(sp_codes)){
  
  my_lm <- lm(cbc[,(3*(i-1))+2] ~ c(1:40))
  my_p <- summary(my_lm)$coefficients[2,4]
  if (my_p < 0.05){
    my_slope <- summary(my_lm)$coefficients[2,1]
  } else{
    my_slope <- 0
  }
  
  my_lm <- lm(cbc[,(3*(i-1))+2] ~ c(1:40))
  my_p <- summary(my_lm)$coefficients[2,4]
  my_slope_all <- summary(my_lm)$coefficients[2,1]
  
  my_lm_atl <- lm(cbc[,(3*(i-1))+3] ~ c(1:40))
  my_p_atl <- summary(my_lm_atl)$coefficients[2,4]
  if (my_p_atl < 0.05){
    my_slope_atl <- summary(my_lm_atl)$coefficients[2,1]
  } else {
    my_slope_atl <- 0
  }
  
  my_lm_atl <- lm(cbc[,(3*(i-1))+3] ~ c(1:40))
  my_p_atl <- summary(my_lm_atl)$coefficients[2,4]
  my_slope_atl_all <- summary(my_lm_atl)$coefficients[2,1]
  
  my_lm_miss <- lm(cbc[,(3*(i-1))+4] ~ c(1:40))
  my_p_miss <- summary(my_lm_miss)$coefficients[2,4]
  if (my_p_miss < 0.05){
    my_slope_miss <- summary(my_lm_miss)$coefficients[2,1]
  } else {
    my_slope_miss <- 0
  }
  
  my_lm_miss <- lm(cbc[,(3*(i-1))+4] ~ c(1:40))
  my_p_miss <- summary(my_lm_miss)$coefficients[2,4]
  my_slope_miss_all <- summary(my_lm_miss)$coefficients[2,1]
  
  cbc_slopes[i,2:7] <- c(my_slope, my_slope_all, 
                        my_slope_atl, my_slope_atl_all,
                        my_slope_miss, my_slope_miss_all)
}



## Repeat this code for all CBC raw Circle files and their
## respective effort files.
## Combine all 65 species into single dataframe.


## Manually add other covariates from the literature:
## Diet percentages: % insect, % fruit, % seed
##   (from Billerman et al. 2020)
## Migration strategy: Moderate distance, Short distance,
##   or Resident (following Soykan et al. 2016)


## This dataset is called "cbc_covs_65species.cvs" in Dryad, 
## and contains columns for both the significant slopes and 
## all slopes for in ENA, ATL, and MISS, the migration
## strategy, and diet % for each species. Use this file
## in the next .R file, "RangeShift_stats.R".


